@@ -1,68 +1,61 @@
 """
+blink1.py -- blink(1) Python library using PyUSB
 
-blink1_pyusb.py -- blink(1) Python library using PyUSB
+Based on blink1hid-demo.py by Aaron Blondeau and the reference Python Plink(1)
+library by Tod E. Kurt, http://thingm.com/
 
 Uses "PyUSB 1.0" to do direct USB HID commands
-
 See: https://github.com/walac/pyusb
 
-Linux (Ubuntu/Debian):
- % sudo apt-get install pip
- % sudo pip install pyusb
- Note: will give "not claimed" error or similar.  Try blink1.py instead
-
-Mac OS X:
- do "brew install libusb" on osx
- or "port install py26-pyusb-devel" on osx
-
-Windows:
- libusb-win32 (inf method) on windows?
-
-Based on blink1hid-demo.py by Aaron Blondeau
-
-2013, Tod E. Kurt, http://thingm.com/
+All platforms:
+ % sudo pip install blink1
 
+Windows Only
+ You will need to manually install libusb-win32
 """
 import logging
 import time
 import sys
-import webcolors
+from contextlib import contextmanager
+
 import usb
 
+import webcolors
+
+
+class BlinkConnectionFailed(RuntimeError):
+    """Raised when we cannot connect to a Blink(1)
+    """
+
 
 log = logging.getLogger(__name__)
 
-report_id = 0x01
-debug_rw = False
-
+REPORT_ID = 0x01
+VENDOR_ID = 0x27b8
+PRODUCT_ID = 0x01ed
 
 class Blink1:
     def __init__(self):
-        self.dev = None
-        return self.find()
+        self.dev = self.find()
+        if not self.dev:
+            raise BlinkConnectionFailed("Could not find an attached Blink(1)")
 
-    def find(self):
-        self.dev = usb.core.find(idVendor=0x27b8, idProduct=0x01ed)
-        if ( self.dev == None ):
+    def close(self):
+        self.dev = None
+
+    @staticmethod
+    def find():
+        dev = usb.core.find(idVendor=VENDOR_ID, idProduct=PRODUCT_ID)
+        if dev == None:
             return None
 
         if not sys.platform.startswith('win'):
-            if ( self.dev.is_kernel_driver_active(0) ):
+            if ( dev.is_kernel_driver_active(0) ):
                 try:
-                    self.dev.detach_kernel_driver(0)
+                    dev.detach_kernel_driver(0)
                 except usb.core.USBError as e:
                     sys.exit("Could not detatch kernel driver: %s" % str(e))
-
-    def enumerate(self):
-        return self.find()
-
-    def open(self):
-        self.close()
-        return self.find()
-
-    def close(self):
-        if self.dev != None:
-            self.dev = None  # FIXME: what's equivalent to: close(self.dev)
+        return dev
 
 
     def notfound(self):
@@ -74,14 +67,13 @@ class Blink1:
         Send USB Feature Report 0x01 to blink(1) with 8-byte payload
         Note: arg 'buf' must be 8 bytes or bad things happen
         """
-        if debug_rw: print
-        "blink1write:" + ",".join('0x%02x' % v for v in buf)
+        log.debug("blink1write:" + ",".join('0x%02x' % v for v in buf))
         if ( self.dev == None ): return self.notfound()
         bmRequestTypeOut = usb.util.build_request_type(usb.util.CTRL_OUT, usb.util.CTRL_TYPE_CLASS,
                                                        usb.util.CTRL_RECIPIENT_INTERFACE)
         self.dev.ctrl_transfer(bmRequestTypeOut,
                                0x09,  # == HID set_report
-                               (3 << 8) | report_id,  # (3==HID feat.report)
+                               (3 << 8) | REPORT_ID,  # (3==HID feat.report)
                                0,
                                buf)
 
@@ -95,50 +87,45 @@ class Blink1:
                                                       usb.util.CTRL_RECIPIENT_INTERFACE)
         buf = self.dev.ctrl_transfer(bmRequestTypeIn,
                                      0x01,  # == HID get_report
-                                     (3 << 8) | report_id,
+                                     (3 << 8) | REPORT_ID,
                                      0,
                                      8)  # == number of bytes to read
-        if debug_rw: print
-        "blink1read: " + ",".join('0x%02x' % v for v in buf)
+        log.debug("blink1read: " + ",".join('0x%02x' % v for v in buf))
         return buf
 
-    def fade_to_rgbn(self, fadeMillis, red, green, blue, ledn):
-        """
-        Command blink(1) to fade to RGB color
-
+    def fade_to_rgbn(self, fade_milliseconds, red, green, blue, ledn):
+        """Command blink(1) to fade to RGB color
         """
         action = ord('c')
-        fadeMillis = int(fadeMillis / 10)
-        th = (fadeMillis & 0xff00) >> 8
-        tl = fadeMillis & 0x00ff
-        buf = [report_id, action, red, green, blue, th, tl, ledn]
+        fade_time = int(fade_milliseconds / 10)
+        th = (fade_time & 0xff00) >> 8
+        tl = fade_time & 0x00ff
+        buf = [REPORT_ID, action, red, green, blue, th, tl, ledn]
         return self.write(buf)
 
-    def fade_to_rgb(self, fadeMillis, red, green, blue):
+    def fade_to_rgb(self, fade_milliseconds, red, green, blue):
         """
         Command blink(1) to fade to RGB color
-
         """
-        return self.fade_to_rgbn(fadeMillis, red, green, blue, 0)
+        return self.fade_to_rgbn(fade_milliseconds, red, green, blue, 0)
 
-    def fade_to_color(self, fadeMillis, color):
+    def fade_to_color(self, fade_milliseconds, color):
+        """
+        Fade the light to a known colour in a
+        :param fade_milliseconds: Duration of the fade in milliseconds
+        :param color: Named color to fade to
+        :return: None
+        """
         red, green, blue = webcolors.name_to_rgb(color)
-        return self.fade_to_rgb(fadeMillis, red, green, blue)
+        return self.fade_to_rgb(fade_milliseconds, red, green, blue)
 
-    def playloop(self, play, startpos, endpos, count):
+    def off(self):
+        """Switch the blink(1) off instantly
         """
-        """
-        buf = [0x01, ord('p'), play, startpos, endpos, count, 0, 0]
-        return self.write(buf)
-
-    def play(self, play, startpos):
-        """
-        """
-        return self.playloop(play, startpos, 0, 0)
+        self.fade_to_color(0, 'black')
 
     def get_version(self):
-        """
-        Get blink(1) firmware version
+        """Get blink(1) firmware version
         """
         if ( self.dev == None ): return ''
         buf = [0x01, ord('v'), 0, 0, 0, 0, 0, 0]
@@ -148,15 +135,20 @@ class Blink1:
         version = (version_raw[3] - ord('0')) * 100 + (version_raw[4] - ord('0'))
         return str(version)
 
-    def get_serialnumber(self):
+    def get_serial_number(self):
+        """Get blink(1) serial number
         """
-        Get blink(1) serial number
-
-        """
-        if ( self.dev == None ): return ''
         return usb.util.get_string(self.dev, 256, 3)
 
-    def get_serialnumbers(self):  # FIXME:
-        seriallist = []
-        seriallist.append(self.get_serialnumber())
-        return seriallist
\ No newline at end of file
+
+@contextmanager
+def blink1(switch_off=True):
+    """Context manager which automatically shuts down the Blink(1)
+    after use.
+    """
+    b1 = Blink1()
+    yield b1
+    if switch_off:
+        b1.off()
+    b1.close()
+
